"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkWellKnownPasswords = checkWellKnownPasswords;
exports.setLinuxPassword = setLinuxPassword;
const child_process_1 = require("child_process");
const WELL_KNOWN_CREDENTIALS = [
    ['root', 'root'],
    ['admin', 'admin'],
    ['admin', 'default'],
    ['root', 'default'],
    ['iob', '2024=smart!'],
    ['pi', 'pi'],
    ['pi', '12345'],
    ['pi', 'raspberry'],
    ['pi', 'default'],
    ['pi', 'pi12345'],
];
// Function to execute 'su' command and provide password
function checkLinuxPassword(login, password) {
    // Check the os
    if (process.platform !== 'linux') {
        console.error('This function is only available on Linux');
        return Promise.resolve(false);
    }
    if (login === 'Password:' || login === 'failure') {
        console.error('This function requires a login name');
        return Promise.resolve(false);
    }
    console.log(`\n[LOG] -------- Check ${login}/${password}`);
    return new Promise((resolve, reject) => {
        try {
            const su = (0, child_process_1.spawn)('su', [login]);
            let closed = false;
            let result = false;
            let responseTimeout = setTimeout(() => {
                closed = true;
                responseTimeout = null;
                su.kill();
            }, 3000);
            function _checkPassword(data) {
                if (!responseTimeout || closed) {
                    return;
                }
                data = data.replace(/\r/g, ' ').replace(/\n/g, ' ').trim();
                console.log(`[STDX] "${data}"`);
                if (data.endsWith(':')) {
                    try {
                        su.stdin?.write(`${password}\n`);
                    }
                    catch {
                        return;
                    }
                    setTimeout(() => {
                        if (!responseTimeout || closed) {
                            return;
                        }
                        console.log(`[LOG] write whoami`);
                        try {
                            su.stdin?.write(`whoami\n`);
                        }
                        catch {
                            // ignore
                        }
                    }, 50);
                }
                else if (data === login) {
                    result = true;
                    su.kill();
                    // todo: add ru, uk, pt,....
                }
                else if (data.toLowerCase().includes('failure') || data.toLowerCase().includes('fehler')) {
                    su.kill();
                }
            }
            su.stdin?.on('error', () => {
                closed = true;
            });
            su.stdout?.on('error', () => {
                closed = true;
            });
            su.stderr?.on('error', () => {
                closed = true;
            });
            su.stdin?.on('finish', () => {
                closed = true;
            });
            su.stdout?.on('finish', () => {
                closed = true;
            });
            su.stderr?.on('finish', () => {
                closed = true;
            });
            su.stdin?.on('close', () => {
                closed = true;
            });
            su.stdout?.on('close', () => {
                closed = true;
            });
            su.stderr?.on('close', () => {
                closed = true;
            });
            // Listen for data on stdout
            su.stdout.on('data', data => {
                if (data && responseTimeout) {
                    _checkPassword(data.toString());
                }
            });
            // Listen for data on stderr
            su.stderr.on('data', data => {
                if (data && responseTimeout) {
                    _checkPassword(data.toString());
                }
            });
            // Listen for the close event
            su.on('close', () => {
                closed = true;
                console.log(`[LOG] -------- closed with result: ${result}\n`);
                if (responseTimeout) {
                    clearTimeout(responseTimeout);
                    responseTimeout = null;
                }
                resolve(result);
            });
        }
        catch (e) {
            closed = true;
            console.error(`[LOG] -------- Error by execution: ${e.message}\n`);
            reject(new Error(e));
        }
    });
}
/** Check if the system has well-known passwords */
async function checkWellKnownPasswords() {
    // Check the os
    if (process.platform !== 'linux') {
        console.error('This function is only available on Linux');
        throw new Error('This function is only available on Linux');
    }
    for (const [login, password] of WELL_KNOWN_CREDENTIALS) {
        if (await checkLinuxPassword(login, password)) {
            return { login, password };
        }
    }
    return null;
}
/*
pi@NanoPi-R5S:/opt$ su pi
Password: [OLD PASSWORD]\n

pi@NanoPi-R5S:/opt$ passwd
Changing password for pi.
Current password: [OLD PASSWORD]\n

New password: [NEW PASSWORD]\n

Retype new password: [NEW PASSWORD]\n

The password has not been changed.

New password:
Retype new password:

You must choose a longer password.

New password:
Retype new password:

passwd: password updated successfully
pi@NanoPi-R5S:/opt$ exit
*/
/*
Ändern des Passworts für pi.
Geben Sie das aktuelle Passwort ein:
passwd: Fehler beim Ändern des Authentifizierungstoken
passwd: Passwort nicht geändert
pi@NanoPi-R5S:~$ passwd pi
Ändern des Passworts für pi.
Geben Sie das aktuelle Passwort ein:
Geben Sie ein neues Passwort ein:
Geben Sie das neue Passwort erneut ein:
passwd: Passwort erfolgreich geändert

 */
const STATES = {
    S_0_SU_WAIT_PROMPT: 0,
    S_1_PASSWD_WAIT_PROMPT_CURRENT_PASSWORD: 1,
    S_2_PASSWD_WAIT_PROMPT_NEW_PASSWORD: 2,
    S_3_PASSWD_WAIT_PROMPT_RETYPE_NEW_PASSWORD: 3,
    S_4_PASSWD_WAIT_RESPONSE: 4,
};
function setLinuxPassword(login, oldPassword, newPassword) {
    return new Promise(resolve => {
        if (WELL_KNOWN_CREDENTIALS.find(item => item[0] === newPassword || item[1] === newPassword)) {
            resolve('New password is well-known too');
            return;
        }
        try {
            const su = (0, child_process_1.spawn)('su', [login]);
            let result = 'Cannot change password';
            let responseTimeout = setTimeout(() => {
                responseTimeout = null;
                result = 'Timeout';
                su.kill();
            }, 3000);
            let state = STATES.S_0_SU_WAIT_PROMPT;
            function _checkPassword(data) {
                data = data.replace(/\r/g, ' ').replace(/\n/g, ' ').trim();
                console.log(`[STDX]: ${data}`);
                if (state === STATES.S_0_SU_WAIT_PROMPT) {
                    // Password: [OLD PASSWORD]\n
                    if (data.endsWith(':')) {
                        console.log(`[LOG]: received request to enter old password`);
                        su.stdin.write(`${oldPassword}\n`);
                        setTimeout(() => {
                            state = STATES.S_1_PASSWD_WAIT_PROMPT_CURRENT_PASSWORD;
                            su.stdin.write(`passwd\n`);
                        }, 50);
                        // todo: add ru, uk, pt,....
                    }
                    else if (data.toLowerCase().includes('failure') || data.toLowerCase().includes('fehler')) {
                        console.log(`[LOG]: received failure message`);
                        result = 'Old password not accepted';
                        su.kill();
                    }
                }
                else if (state === STATES.S_1_PASSWD_WAIT_PROMPT_CURRENT_PASSWORD) {
                    // Changing password for pi.
                    // Current password: [OLD PASSWORD]\n
                    if (data.endsWith(':')) {
                        console.log(`[LOG]: received request to enter new password`);
                        state = STATES.S_2_PASSWD_WAIT_PROMPT_NEW_PASSWORD;
                        su.stdin.write(`${oldPassword}\n`);
                    }
                }
                else if (state === STATES.S_2_PASSWD_WAIT_PROMPT_NEW_PASSWORD) {
                    // New password: [NEW PASSWORD]\n
                    if (data.endsWith(':')) {
                        console.log(`[LOG]: received request to enter new password`);
                        state = STATES.S_3_PASSWD_WAIT_PROMPT_RETYPE_NEW_PASSWORD;
                        su.stdin.write(`${newPassword}\n`);
                    }
                }
                else if (state === STATES.S_3_PASSWD_WAIT_PROMPT_RETYPE_NEW_PASSWORD) {
                    // Retype new password: [NEW PASSWORD]\n
                    if (data.endsWith(':')) {
                        console.log(`[LOG]: received request to repeat new password`);
                        state = STATES.S_4_PASSWD_WAIT_RESPONSE;
                        su.stdin.write(`${newPassword}\n`);
                    }
                }
                else if (state === STATES.S_4_PASSWD_WAIT_RESPONSE) {
                    // todo: add ru, uk, pt,....
                    if (data.toLowerCase().includes('successfully') || data.toLowerCase().includes('erfolgreich')) {
                        result = true;
                    }
                    else {
                        // failure
                        result = data;
                    }
                    su.kill();
                }
            }
            // Listen for data on stdout
            su.stdout.on('data', data => _checkPassword(data.toString()));
            // Listen for data on stderr
            su.stderr.on('data', data => _checkPassword(data.toString()));
            // Listen for the close event
            su.on('close', () => {
                responseTimeout && clearTimeout(responseTimeout);
                responseTimeout = null;
                resolve(result);
            });
        }
        catch (e) {
            console.error(`Cannot change password: ${e.toString()}`);
            throw new Error(e);
        }
    });
}
//# sourceMappingURL=checkLinuxPass.js.map